"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path = require('path');
const TransmuteFramework_1 = require("../../TransmuteFramework");
const eventStoreArtifacts = require('../../../build/contracts/EventStore');
const eventStoreFactoryArtifacts = require('../../../build/contracts/EventStoreFactory');
const { getCachedReadModel } = TransmuteFramework_1.default.EventStore;
const reducer_1 = require("./reducer");
const _ = require("lodash");
const { web3, EventStoreContract, EventStoreFactoryContract, EventStore, TransmuteIpfs } = TransmuteFramework_1.default.init({
    env: 'testrpc',
    esa: eventStoreArtifacts,
    esfa: eventStoreFactoryArtifacts
});
const createEventStore = (factory, fromAddress) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const { tx, events } = yield TransmuteFramework_1.default.EventStore.EventStoreFactory.createEventStore(factory, fromAddress);
    const newEsAddress = events[0].AddressValue;
    return newEsAddress;
});
const syncEventStore = (bindingModel) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    let { contractAddress, fromAddress } = bindingModel;
    let eventStore = yield EventStoreContract.at(contractAddress);
    let updatedReadModel = yield getCachedReadModel(contractAddress, eventStore, fromAddress, reducer_1.readModel, reducer_1.reducer);
    return updatedReadModel;
});
const writeEvent = (bindingModel) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    let { contractAddress, fromAddress, event } = bindingModel;
    event = _.cloneDeep(event);
    let eventStore = yield EventStoreContract.at(contractAddress);
    let events = yield TransmuteFramework_1.default.EventStore.writeTransmuteCommand(eventStore, fromAddress, event);
    let updatedReadModel = yield getCachedReadModel(contractAddress, eventStore, fromAddress, reducer_1.readModel, reducer_1.reducer);
    return updatedReadModel;
});
exports.default = (vorpal) => {
    vorpal
        .command('eventstore create', 'Create an EventStore')
        .action((args, callback) => {
        web3.eth.getAccounts((err, addresses) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (err) {
                throw err;
            }
            if (args.options.from === undefined) {
                args.options.from = addresses[0];
            }
            let fromAddress = args.options.from;
            let factory = yield EventStoreFactoryContract.deployed();
            let contractAddress = yield createEventStore(factory, fromAddress);
            console.log('🎁  ' + contractAddress + ' EventStore created...');
            callback();
        }));
    });
    vorpal
        .command('eventstore show', 'Show an EventStore')
        .option('-f, --from <from>', 'from address')
        .option('-c, --contractAddress <contractAddress>', 'contractAddress...')
        .types({
        string: ['contractAddress']
    })
        .action((args, callback) => {
        web3.eth.getAccounts((err, addresses) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (err) {
                throw err;
            }
            if (args.options.from === undefined) {
                args.options.from = addresses[0];
            }
            let fromAddress = args.options.from;
            let bindingModel = {
                contractAddress: args.options.contractAddress,
                fromAddress: args.options.from
            };
            let updatedReadModel = yield syncEventStore(bindingModel);
            console.log(updatedReadModel);
            callback();
        }));
    });
    vorpal
        .command('eventstore write', 'Write an Event to an EventStore')
        .option('-f, --from <from>', 'from address')
        .option('-c, --contractAddress <contractAddress>', 'contractAddress...')
        .option('-t, --type <type>', 'event type')
        .option('-p, --payload <payload>', 'event payload')
        .types({
        string: ['contractAddress']
    })
        .action((args, callback) => {
        web3.eth.getAccounts((err, addresses) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (err) {
                throw err;
            }
            if (args.options.from === undefined) {
                args.options.from = addresses[0];
            }
            let fromAddress = args.options.from;
            let bindingModel = {
                contractAddress: args.options.contractAddress,
                fromAddress: args.options.from,
                event: {
                    type: args.options.type,
                    payload: args.options.payload
                }
            };
            let updatedReadModel = yield writeEvent(bindingModel);
            console.log(updatedReadModel);
            callback();
        }));
    });
    return vorpal;
};
