export declare module EventTypes {
    interface IBigNumber {
        toNumber: () => number;
    }
    interface IEsEventFromTruffle {
        Id: IBigNumber;
        Type: string;
        Version: string;
        ValueType: string;
        IsAuthorized: boolean;
        PermissionDomain: string;
        AddressValue: string;
        UIntValue: IBigNumber;
        Bytes32Value: string;
        TxOrigin: string;
        Created: IBigNumber;
    }
    interface IEsEvent {
        Id?: number;
        Type: string;
        Version: string;
        ValueType: string;
        IsAuthorized: boolean;
        PermissionDomain: string;
        AddressValue: string;
        UIntValue: number;
        Bytes32Value: string;
        StringValue: string;
        TxOrigin?: string;
        Created?: number;
    }
    interface ITransmuteEventMeta {
        id: number;
        version: string;
        txOrigin: string;
        created: number;
        path?: string;
    }
    interface ITransmuteEvent {
        type: string;
        payload: any;
        error?: any;
        meta: ITransmuteEventMeta;
    }
    interface ITransmuteCommandMeta {
        handlers: Array<string>;
    }
    interface ITransmuteCommand {
        type: string;
        payload: any;
        meta?: ITransmuteCommandMeta;
    }
    interface ITransaction {
        tx: string;
        receipt: any;
        logs: any[];
    }
    interface ITransmuteCommandResponse {
        events: Array<ITransmuteEvent>;
        transactions: Array<ITransaction>;
    }
    interface IReadModel {
        lastEvent: number;
        readModelType: string;
        readModelStoreKey: string;
        contractAddress: string;
        model: any;
    }
    const toAscii: (value: any) => any;
    const guessType: (value: any) => "Address" | "UInt" | "String";
    const EsEvent = "EsEvent";
    const EsEventSchema: {
        Id: string;
        Type: string;
        Version: string;
        ValueType: string;
        IsAuthorized: string;
        PermissionDomain: string;
        AddressValue: string;
        UIntValue: string;
        Bytes32Value: string;
        StringValue: string;
        TxOrigin: string;
        Created: string;
    };
    const TruffleEventSchema: {
        [x: string]: {
            Id: string;
            Type: string;
            Version: string;
            ValueType: string;
            IsAuthorized: string;
            PermissionDomain: string;
            AddressValue: string;
            UIntValue: string;
            Bytes32Value: string;
            StringValue: string;
            TxOrigin: string;
            Created: string;
        };
    };
    const getPropFromTruffleEventSchemaType: (key: string, schema: any, truffleValue: any) => any;
    const getEsEventFromEsEventValues: (eventValues: any) => IEsEventFromTruffle;
    const getEsEventFromEsEventWithTruffleTypes: (eventType: string, solEvent: IEsEventFromTruffle) => IEsEvent;
    const getValueType: (es: any) => any;
    const convertMeta: (esEvent: IEsEvent) => any;
    const esEventToTransmuteEvent: (esEvent: IEsEvent) => Promise<ITransmuteEvent>;
    const convertCommandToEsEvent: (transmuteCommand: ITransmuteCommand) => IEsEvent;
    const flatten: (data: any) => {};
    const unflatten: (data: Object) => Object;
    const eventsFromTransaction: (tx: any) => any;
    const reconstructTransmuteEventsFromTxs: (txs: ITransaction[]) => any[];
}
