"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const EventStore_1 = require("./EventStore/EventStore");
const EventTypes_1 = require("./EventStore/EventTypes/EventTypes");
const Persistence_1 = require("./EventStore/Persistence/Persistence");
const TransmuteIpfs_1 = require("./TransmuteIpfs/TransmuteIpfs");
const Web3 = require('web3');
const contract = require('truffle-contract');
let web3;
const eventStoreArtifacts = require('../build/contracts/EventStore');
const eventStoreFactoryArtifacts = require('../build/contracts/EventStoreFactory');
const config = {
    env: 'testrpc',
    esa: eventStoreArtifacts,
    esfa: eventStoreFactoryArtifacts,
};
class TransmuteFramework {
    constructor() {
        this.EventStoreFactoryContract = null;
        this.EventStoreContract = null;
        this.EventStore = null;
        this.config = config;
        this.web3 = null;
        this.TransmuteIpfs = TransmuteIpfs_1.TransmuteIpfs;
        this.EventTypes = EventTypes_1.EventTypes;
        this.Persistence = Persistence_1.Persistence;
        this.init = (confObj = config) => {
            this.config = confObj;
            switch (this.config.env) {
                case 'testrpc':
                    web3 = new Web3(new Web3.providers.HttpProvider('http://localhost:8545'));
                    break;
                case 'parity':
                    web3 = new Web3(new Web3.providers.HttpProvider('http://localhost:8545'));
                    break;
                case 'infura':
                    web3 = new Web3(new Web3.providers.HttpProvider('https://ropsten.infura.io'));
                    break;
                case 'metamask':
                    web3 = window.web3;
                    break;
            }
            this.web3 = web3;
            if (this.web3) {
                this.EventStoreFactoryContract = contract(this.config.esfa);
                this.EventStoreFactoryContract.setProvider(this.web3.currentProvider);
                this.EventStoreContract = contract(this.config.esa);
                this.EventStoreContract.setProvider(this.web3.currentProvider);
            }
            else {
                console.warn('web3 is not available, install metamask.');
            }
            let ipfsConfig = confObj.ipfsConfig || {
                host: 'localhost',
                port: '5001',
                options: {
                    protocol: 'http'
                }
            };
            this.TransmuteIpfs = TransmuteIpfs_1.TransmuteIpfs.init(ipfsConfig);
            this.EventStore = new EventStore_1.EventStore(this);
            return this;
        };
    }
}
exports.TransmuteFramework = TransmuteFramework;
exports.default = new TransmuteFramework();
